function [A, B, C] = Angles_ABC_a_partir_du_CosinusDirecteur(X)
% Angles_ABC_a_partir_du_CosinusDirecteur
%
% Cette fonction permet de disposer des angles A, B, C, dfinis selon la
% convention (z,y,x) du KUKA KR3 correspondant  une matrice d'orientation X.
%
% [A, B, C] = Angles_ABC_a_partir_du_CosinusDirecteur(X)
%
% est tel que : 'X' est le paramtre d'entre correspondant  la matrice,
% de dimension (3,3), d'orientation ; 'A', 'B', 'C' sont les angles
% Roll (roulis), Pitch (tangage), Yaw (lacet), exprims en radians.
%
% Singularit si B = 90 ou -90, c--d, si s_x et s_y sont nuls !

% Dfinition des vecteurs s, n et a : X =[s n a]
s_x = X(1,1); n_x = X(1,2); a_x = X(1,3); 
s_y = X(2,1); n_y = X(2,2); a_y = X(2,3); 
s_z = X(3,1); % n_z = A(3,2) et a_z = A(3,3) ne sont pas utiliss 

A = atan2(s_y, s_x);
% A_prime = A + pi : autre solution non calcule !

B = atan2(-s_z, cos(A)*s_x + sin(A)*s_y);

C = atan2(sin(A)*a_x - cos(A)*a_y, -sin(A)*n_x + cos(A)*n_y);
end