% Calcul d'une trajectoire rectiligne du TCP (Tool Center Point), situ au
% centre de la flasque du robot, dans l'espace des tches
% Application au robot KUKA KR3 : trajectoire avec, ou sens, changement 
% d'orientation de l'outil (selon la valeur de $ORI_TYPE = #VAR ou #CONSTANT)

% La mthode utilise pour l'orientation est base sur la dtermination
% d'un vecteur unitaire (U) et d'un angle (gamma) permettant d'obtenir la
% matrice de rotation finale (A_f)  partir de celle initiale (A_i) (suite
%  une rotation autour de U d'un angle gamma)

% Prise de mesure du TCP sur le smartPAD du robot (ou via le progiciel 
% Kuka SimPro) :
% + du point initial (P_Ini) :
%           x_I, y_I, z_I <=> position (en mm)
%           A_I, B_I, C_I <=> orientation (en degr, selon la convention ZYX)
% + du point final (P_Fin) :
%           x_F, y_F, z_F <=> position (en mm)
%           A_F, B_F, C_F <=> orientation (en degr, selon la convention ZYX)
  
% cf. JL4.vcmx : changement d'orientation (gamma = 180, $ORI_TYPE = #VAR)
  x_I=242; y_I=-397; z_I=527;
  A_I=-66; B_I=0; C_I=-10;  
  x_F=292; y_F=-318; z_F=420;
  A_F=-66; B_F=0; C_F=170;
  
% % cf. JL3.vcmx : pas de changement d'orientation (gamma = 0, $ORI_TYPE = #CONSTANT)
%   x_I=215; y_I=-300; z_I=300;
%   A_I=-130; B_I=12; C_I=-150;
%   x_F=360; y_F=-310; z_F=340;
%   A_F=-130; B_F=12; C_F=-150;

% Calcul de la matrice de transformation homogne X_i correspondant au point
% initial P_Ini du TCP (avec X_i = [A_i P_i ; [0 0 0 1]])
% -> matrice de rotation (3 x 3) initiale du TCP : A_i = [s_i; n_i; a_i]
  A_i = CosinusDirecteur_a_partir_des_angles_ABC(A_I*pi/180, B_I*pi/180, C_I*pi/180);
  A_i = A_i(1:3,1:3);
  s_i = A_i(:,1); n_i = A_i(:,2); a_i = A_i(:,3);
% -> position initiale du TCP : P_i = [x_I; y_I; z_I]
  P_i = [x_I; y_I; z_I];
% X_i = [A_i P_i; 0 0 0 1] (dimension (4, 4))
  X_i = [A_i P_i; [0 0 0 1]];

% Calcul de la matrice de transformation homogne X_f correspondant au point
% final P_Fin du TCP (avec X_f = [A_f P_f ; [0 0 0 1]])
% -> matrice de rotation (3 x 3) finale du TCP : A_f = [s_f; n_f; a_f]
  A_f = CosinusDirecteur_a_partir_des_angles_ABC(A_F*pi/180, B_F*pi/180, C_F*pi/180);
  A_f = A_f(1:3,1:3);
  s_f = A_f(:,1); n_f = A_f(:,2); a_f = A_f(:,3);
% -> position finale du TCP : P_f = [x_F; y_F; z_F]
  P_f = [x_F; y_F; z_F];
% X_f = [A_f P_f; 0 0 0 1] (dimension (4, 4))
  X_f = [A_f P_f; [0 0 0 1]];

% Dtermination du vecteur U et de l'angle gamma  partir de l'quation :
% A_i Rot(U, gamma) = A_f
  % -> calcul de gamma
    % notation : C_gamma pour cos(gamma), S_gamma pour sin(gamma)
    C_gamma = 1/2 * (dot(s_i,s_f) + dot(n_i,n_f) + dot(a_i,a_f) - 1);
    S_gamma = 1/2 * sqrt((dot(a_i,n_f)-dot(n_i,a_f))^2 + ...
                         (dot(s_i,a_f)-dot(a_i,s_f))^2 + ...
                         (dot(n_i,s_f)-dot(s_i,n_f))^2);
    gamma = atan2(S_gamma, C_gamma);
    gamma_degree = gamma * 180/pi   % affichage de gamma en degr
  % calcul du vecteur U (plusieurs cas sont  prvoir selon que sin(gamma)
  % est proche ( epsilon prs) de 0
    eps = .0001;  % valeur de epsilon
    % cas (gnral) o sin(gamma) n'est pas proche de 0
    if abs(S_gamma) >= eps
      U = (1/(2*S_gamma)) * [dot(a_i,n_f)-dot(n_i,a_f) ... 
          dot(s_i,a_f)-dot(a_i,s_f) dot(n_i,s_f)-dot(s_i,n_f)]'
    % cas o sin(gamma) est proche de 0 car gamma est proche de 0
    elseif C_gamma > 0  % cos(gamma) proche de 1 car sin(gamma) proche de 0
      U = [1 0 0]'  % gamma nul -> U est choisi de manire arbitraire
      % (pris gal  x_0), on a : Rot_U_gamma(U, gamma) = Id(3,3)
    % cas o sin(gamma) est proche de 0 car gamma est proche de Pi
    elseif C_gamma < 0  && abs(dot(s_i,s_f)) >= eps % cos(gamma) proche de
        % -1 (car sin(gamma) proche de 0) et U_x <> 0
      U_x = sqrt((dot(s_i,s_f)+1)/2);
      U = [U_x  dot(n_i,s_f)/(2*U_x)  dot(a_i,s_f)/(2*U_x)]'
    elseif C_gamma < 0  && abs(dot(n_i,n_f)) >= eps % cos(alpha) proche de
        % -1 (car sin(gamma) proche de 0) et U_y <> 0
      U_y = sqrt((dot(n_i,n_f)+1)/2);
      U = [dot(s_i,n_f)/(2*U_y)  U_y  dot(a_i,n_f)/(2*U_y)]'
    elseif C_gamma < 0  && abs(dot(a_i,a_f)) >= eps % cos(gamma) proche de
        % -1 (car sin(gamma) proche de 0) et U_z <> 0
      U_z = sqrt((dot(a_i,a_f)+1)/2);
      U = [dot(s_i,a_f)/(2*U_z)  dot(n_i,a_f)/(2*U_z)  U_z]'
    else % problme pour calculer le vecteur U quand gamma est proche de Pi !!
      fprintf('problme pour calculer le vecteur U quand gamma est proche de Pi !!\n')
    end

% Calcul du vecteur P(t) et de la matrice de rotation A(t) entre les
% instants : t_initial=0(=t(n=1)) et t_final=t_f(=t(n=N+1). Le calcul est
% ralis  intervalles de temps rguliers : N+1 points sont calculs
  t_f = 1; N=10;   % prendre N+1 >= 2, c'est--dire, N >= 1
% Calcul des N+1 points
for n = 1:N+1
  % Calcul du temps t(n) correspondant au point (n) en cours, soit t(n) =
  % (n-1)/N * t_f, on a : t(1) = 0, t(2) = t_f/N, t(3) = 2*t_f/N, ..., t(N+1) = t_f
    t(n) = ((n-1)/N)*t_f;
  % Calcul de la fonction d'interpolation linaire correspondant au point (n) en cours,
  % soit r(n) = t(n)
  % remarque : on a  r(t=0)=0 <= r(t) <= r(t=t_f)=1
    r(n) = t(n);
  % Calcul du point P(n) correspondant au point (n) en cours, soit
  % P(:,n) = ????
  
  % Calcul de la matrice de rotation A(n) correspondant au point en cours, 
  % A(:,:,n) = ????
  
  % Calcul des angles A_z, B_y, C_x (en degr) selon la convention ZYX
  % correspondant  la matrice A(n), ces angles tant utiliss par la suite
  % pour la reprsentation graphique
    [A_z(n), B_y(n), C_x(n)] = Angles_ABC_a_partir_du_CosinusDirecteur(A(:,:,n));
    A_z(n)=180/pi*A_z(n); B_y(n)=180/pi*B_y(n); C_x(n) = 180/pi* C_x(n);
end

% Affichage de l'volution des vecteurs x et z des points calculs au cours du temps
% entre t_initial=0 et t_final=t_f
%
% Affichage des vecteurs x_0, y_0, z_0 du repre R_0
% En fait la taille des vecteurs reprsents dans la figure est multiplie
% par 50 afin qu'ils soient plus visibles
  figure(1)
  quiver3(0,0,0,50,0,0,'k-','linewidth',3,'MaxHeadSize',.6,'color','r')
  hold on
  title('affichage dans R_0 des composantes x (rouge) et z (bleu) des points calculs')
  xlabel('x_{0}','FontSize',12,'FontWeight','bold')
  quiver3(0,0,0,0,50,0,'k-','linewidth',3,'MaxHeadSize',.6,'color','g')
  ylabel('y_{0}','FontSize',12,'FontWeight','bold')
  quiver3(0,0,0,0,0,50,'k-','linewidth',3,'MaxHeadSize',.6,'color','b')
  zlabel('z_{0}','FontSize',12,'FontWeight','bold')
 % 
 % Affichage de l'volution des vecteurs x et z des points au cours du temps
 % (le vecteur y n'est pas reprsent afin que la figure soit plus lisible)
   for n = 1:N+1
     % vecteur x (fois 50) en rouge
     quiver3(P(1,n),P(2,n),P(3,n),A(1,1,n)*50,A(2,1,n)*50,A(3,1,n)*50,'color','r')
     % vecteur z (fois 50) en bleu
     quiver3(P(1,n),P(2,n),P(3,n),A(1,3,n)*50,A(2,3,n)*50,A(3,3,n)*50,'color','b')
   end
  hold off
 % 
 % Affichage de l'volution des angles A, B, C au cours du temps
 % rem : les courbes sont interpoles afin d'accder  des mesures intermdiaires
   figure(2), plot(interp(t,9),interp(A_z,9),t,A_z,'+'), title ('angle A (en )')
   % codes pour changer les intituls des coordonnes du 'data cursor'
   % (accessible dans la figure(2)), utilisation galement de la fonction
   % 'myupdatefcnA'
   dcm_objA = datacursormode(figure(2));
   set(dcm_objA,'UpdateFcn',@myupdatefcnA)
   %
   figure(3), plot(interp(t,9),interp(B_y,9),t,B_y,'+'), title ('angle B (en )')
   % codes pour changer les intituls des coordonnes du 'data cursor'
   % (accessible dans la figure(3)), utilisation galement de la fonction
   % 'myupdatefcnB'
   dcm_objB = datacursormode(figure(3));
   set(dcm_objB,'UpdateFcn',@myupdatefcnB)
   %
   figure(4), plot(interp(t,9),interp(C_x,9),t,C_x,'+'), title ('angle C (en )')
   % codes pour changer les intituls des coordonnes du 'data cursor'
   % (accessible dans la figure(4)), utilisation galement de la fonction
   % 'myupdatefcnC'
   dcm_objC = datacursormode(figure(4));
   set(dcm_objC,'UpdateFcn',@myupdatefcnC)
    
 % Fonctions permettant dans les figures prcdentes reprsentant l'volution
 % des angles A, B, C d'afficher l'angle et le temps lorsque le curseur est utilis 
   function txt = myupdatefcnA(~,event_obj)  % angle A
     pos = get(event_obj,'Position');
     txt = {['Time : ',num2str(pos(1))],...
       ['A : ',num2str(pos(2))]};
   end
   % 
   function txt = myupdatefcnB(~,event_obj)  % angle B
     pos = get(event_obj,'Position');
     txt = {['Time : ',num2str(pos(1))],...
       ['B : ',num2str(pos(2))]};
   end
   % 
   function txt = myupdatefcnC(~,event_obj)  % angle C
     pos = get(event_obj,'Position');
     txt = {['Time : ',num2str(pos(1))],...
       ['C : ',num2str(pos(2))]};
   end
